from colorama import Fore, Back, Style
from address_book import AddressBook, Record
from error_decorator import input_error
from output_formatter import format_success, format_error


@input_error
def contacts_handlers(command, contacts, *arguments):
    no_args_handlers_map = {
        'hello': hello_handler,
        'help': help_handler,
    }

    handlers_map = {
        'add': add_contact_handler,
        'add-email': add_email_handler,
        'change-email': change_email_handler,
        'set-address': set_address,
        'change': change_phone_handler,
        'phone': get_contact_handler,
        'all': get_all_contacts_handler,
        'add-birthday': add_birthday_handler,
        'show-birthday': get_birthday_handler,
        'birthdays': get_all_birthdays_handler,
    }

    if command in no_args_handlers_map:
        return no_args_handlers_map[command]()

    return handlers_map[command](contacts, *arguments)


def hello_handler():
    return 'Hello, how can I help you?'


def help_handler():
    return f'''Possible commands:
{Fore.LIGHTWHITE_EX}{Back.BLUE}help{Style.RESET_ALL} - prints list of available commands
{Fore.LIGHTWHITE_EX}{Back.BLUE}hello{Style.RESET_ALL} - prints a greeting 
{Fore.LIGHTWHITE_EX}{Back.BLUE}add [name] [phone number]{Style.RESET_ALL} - create a contact with a phone number
{Fore.LIGHTWHITE_EX}{Back.BLUE}add-email [name] [email] [is_primary]{Style.RESET_ALL} - Add or change email in contact, or create new contact with email if not exists
{Fore.LIGHTWHITE_EX}{Back.BLUE}change-email [name] [old_email_value] [new_email_value] [is_primary]{Style.RESET_ALL} - Add or change email in contact, or create new contact with email if not exists
{Fore.LIGHTWHITE_EX}{Back.BLUE}set-address [name] [address]{Style.RESET_ALL} - set address into contact
{Fore.LIGHTWHITE_EX}{Back.BLUE}change [name] [phone number]{Style.RESET_ALL} - changes a contact phone number 
{Fore.LIGHTWHITE_EX}{Back.BLUE}phone [name]{Style.RESET_ALL} - prints contacts phone number
{Fore.LIGHTWHITE_EX}{Back.BLUE}all{Style.RESET_ALL} - prints all contacts
{Fore.LIGHTWHITE_EX}{Back.BLUE}add-birthday [name] [birthday]{Style.RESET_ALL} - adds birthday to a contact
{Fore.LIGHTWHITE_EX}{Back.BLUE}show-birthday [name]{Style.RESET_ALL} - prints contact's birthday
{Fore.LIGHTWHITE_EX}{Back.BLUE}birthdays{Style.RESET_ALL} - prints all birthdays
{Fore.LIGHTWHITE_EX}{Back.BLUE}close{Style.RESET_ALL} або {Fore.YELLOW}{Back.BLUE}exit{Style.RESET_ALL} - terminates a program
    '''


@input_error
def add_contact_handler(contacts: AddressBook, name: str, phone: str):
    name = name.lower().capitalize()

    try:
        record = contacts.find(name)
    except ValueError:
        record = Record(name)

    record.add_phone(phone)
    contacts[name] = record

    return format_success('Contact added')


@input_error
def add_email_handler(contacts: AddressBook, name: str, email: str, is_primary: str = "False"):
    name = name.lower().capitalize()
    primary = parser_bool_from_str(is_primary)

    record = contacts.find(name)
    if record is None:
        record = Record(name)

    record.add_email(email, primary)
    contacts[name] = record

    return format_success('Email added')


@input_error
def change_phone_handler(contacts: AddressBook, name: str, phone: str):
    name = name.lower().capitalize()
    try:
        record = contacts.find(name)
        record.edit_phone(record.phones[0], phone)

        return format_success('Contact updated.')
    except ValueError:
        return format_error('Contact not found.')


@input_error
def change_email_handler(contacts: AddressBook, name: str, old_email: str, new_email: str, is_primary: str):
    name = name.lower().capitalize()
    primary = parser_bool_from_str(is_primary)

    try:
        record = contacts.find(name)
        record.change_email(old_email, new_email, primary)

        return format_success('Email updated.')
    except ValueError as error:
        return format_error(error)


@input_error
def set_address(contacts: AddressBook, name: str, *args):
    try:
        record = contacts.find(name)
        record.set_address(' '.join(args))

        return format_success('The address is set.')
    except ValueError as error:
        return format_error(error)


@input_error
def get_contact_handler(contacts: AddressBook, name: str, *args):
    try:
        return contacts.find(name.lower().capitalize())
    except ValueError:
        return format_error('Contact not found.')


def get_all_contacts_handler(contacts: AddressBook, *args):
    if len(contacts) == 0:
        return format_error('AddressBook is empty.')
    return '\n'.join(map(lambda name: f'{name}: {contacts[name]}', contacts.keys()))


def add_birthday_handler(contacts: AddressBook, name: str, birthday: str):
    try:
        contacts.find(name.lower().capitalize()).add_birthday(birthday)
        return format_success('Contact updated.')
    except ValueError as error:
        return format_error(error)


def get_birthday_handler(contacts: AddressBook, name: str, *args):
    try:
        return contacts.find(name.lower().capitalize()).birthday
    except ValueError:
        return format_error('Contact not found.')


def get_all_birthdays_handler(contacts: AddressBook, *args):
    return '\n'.join(map(lambda name: f'{name}: {contacts[name].birthday}', contacts.keys()))


def parser_bool_from_str(val: str) -> bool:
    return val.lower() == 'true'



from address_book import AddressBook
from command_handler import hello_handler, help_handler, contacts_handlers
from error_decorator import input_error
from file_storage import load_data, save_data
from output_formatter import format_success
from prompt_toolkit import prompt
from prompt_toolkit.completion import WordCompleter


@input_error
def parse_command(input_sting: str):
    command, *arguments = input_sting.split()

    return command, *arguments


def run():
    try:
        contacts = load_data()
    except FileNotFoundError:
        contacts = AddressBook()

    print(hello_handler())
    print(help_handler())

    # Предполагаемый список доступных команд
    commands = [ 'help', 'hello', 'add', 'change', 'phone', 'all', 'add-birthday', 'show-birsthday', 'birthdays','close','exit']

    # Создание объекта WordCompleter с доступными командами
    command_completer = WordCompleter(commands, ignore_case=True)

    try:
        while True:
            user_input = prompt('>> ', completer=command_completer)
            command, *arguments = parse_command(user_input)

            if command in ['exit', 'close']:
                print(format_success('Good bye!'))
                break

            print(contacts_handlers(command, contacts, *arguments))

    except KeyboardInterrupt:
        print(format_success('\nGood bye!'))
    finally:
        save_data(contacts)





import pickle


def save_data(data_dict, filename="storage.bin"):
    with open(filename, "wb") as f:
        pickle.dump(data_dict, f)


def load_data(filename="storage.bin"):
    with open(filename, "rb") as f:
        return pickle.load(f)



from re import match

from record.field import Field


class Email(Field):

    def __init__(self, value: str, primary: bool):
        if not match(r'^.+@.+\..+$', value):
            raise ValueError(
                'Email should be in format {one or more any char}@{one or more any char}.{one or more any char}'
            )

        self.primary = primary
        super().__init__(value=value)

    def __str__(self):
        return f'{{value: {super().__str__()}, primary:{self.primary} }}'



from re import match

from record.field import Field


class Phone(Field):
    # реалізація класу
    def __init__(self, value):
        if not match(r'^\d{10}$', value):
            raise ValueError('Phone number must be 10 digits')

        super().__init__(value=value)



from collections import UserDict
from record.record import Record


class AddressBook(UserDict):
    def add_record(self, record: Record):
        self.data[str(record.name)] = record

    def find(self, name: str) -> Record:
        if name not in self.data.keys():
            raise ValueError('Record not found')

        return self.data[name]

    def delete(self, name: str):
        if name not in self.data.keys():
            raise ValueError('Record not found')

        self.data.pop(name)
